{% load i18n fb_tags %}
<div id="changelist-filter">
    <h2>{% trans 'Filter' %}</h2>

    <h3>{% trans "By Date" %}</h3>
    <ul>
        <li{% if not query.filter_date %} class="selected"{% endif %}>
        <a href="{% query_string "" "filter_date,p" %}">{% trans "Any Date" %}</a></li>
        <li{% if query.filter_date == "today" %} class="selected"{% endif %}>
        <a href="{% query_string "" "filter_date,p" %}&amp;filter_date=today">{% trans "Today" %}</a></li>
        <li{% if query.filter_date == "past7days" %} class="selected"{% endif %}>
        <a href="{% query_string "" "filter_date,p" %}&amp;filter_date=past7days">{% trans "Past 7 days" %}</a></li>
        <li{% if query.filter_date == "thismonth" %} class="selected"{% endif %}>
        <a href="{% query_string "" "filter_date,p" %}&amp;filter_date=thismonth">{% trans "This Month" %}</a></li>
        <li{% if query.filter_date == "thisyear" %} class="selected"{% endif %}>
        <a href="{% query_string "" "filter_date,p" %}&amp;filter_date=thisyear">{% trans "This year" %}</a></li>
    </ul>

    <h3>{% trans "By Type" %}</h3>
    <ul>
        <li{% if not query.filter_type %} class="selected"{% endif %}>
        <a href="{% query_string "" "filter_type,p" %}">{% trans "All" %}</a></li>
        <li{% if query.filter_type == 'Folder' %} class="selected"{% endif %}>
        <a href="{% query_string "" "filter_type,p" %}&amp;filter_type=Folder">{% trans "Folder" %}</a></li>
        {% for extension in settings_var.EXTENSIONS %}
        <li{% if query.filter_type == extension %} class="selected"{% endif %}>
        <a href="{% query_string "" "filter_type,p" %}&amp;filter_type={{ extension }}">{% trans extension %}</a></li>
        {% endfor %}
    </ul>
</div>
